use serde::Deserialize;
use serde_hex::{CompactPfx, SerHex, StrictPfx};

/// Register group.
#[derive(Debug, Deserialize)]
pub struct Module {
    #[serde(rename = "@name")]
    pub name: String,
    #[serde(rename = "@addr", with = "SerHex::<CompactPfx>")]
    pub addr: u32,
    #[serde(rename = "@size")]
    pub size: String,
    #[serde(rename = "@amod")]
    pub amod: Option<ModuleAmodValue>,
    #[serde(rename = "@type")]
    pub r#type: Option<DataType>,
    #[serde(default)]
    pub child_bitstring: Vec<BitString>,
    #[serde(rename = "$value")]
    pub child_elements: Vec<ModuleElements>,
}

#[derive(Debug, Deserialize)]
pub enum ModuleAmodValue {
    A16,
    A24,
    A32,
    CRCSR,
    USER1,
    USER2,
}

#[derive(Debug, Deserialize)]
pub enum DataType {
    /// 32bit data type
    D32,
}

#[derive(Debug, Deserialize)]
pub struct BitString {
    // TODO
}

#[derive(Debug, Deserialize)]
pub struct Block {
    #[serde(rename = "@name")]
    pub name: String,
    #[serde(rename = "@addr")]
    pub addr: Option<usize>,
    #[serde(rename = "@type")]
    pub r#type: Option<DataType>,
    #[serde(rename = "@modf")]
    pub modf: Option<RwSpecifier>,
    #[serde(rename = "@size")]
    pub size: String,
    #[serde(rename = "@amod")]
    pub amod: Option<ModuleAmodValue>,
    #[serde(default)]
    pub child_bitstring: Vec<BitString>,
    #[serde(rename = "$value")]
    pub child_elements: Vec<ModuleElements>,
    // TODO
}
#[derive(Debug, Deserialize)]
pub struct Memory {
    // TODO
}
#[derive(Debug, Deserialize)]
pub struct Register {
    // TODO
}
#[derive(Debug, Deserialize)]
pub struct Fifo {
    // TODO
}

#[derive(Debug, Deserialize)]
pub enum ModuleElements {
    #[serde(rename = "block")]
    Block(Block),
    Memory(Memory),
    Register(Register),
    Fifo(Fifo),
}

#[derive(Debug, Deserialize)]
pub enum RwSpecifier {}
